package test;


import java.util.HashMap;
import java.util.Map;
import java.util.Random;
import java.util.Timer;
import java.util.TimerTask;
import org.apache.log4j.Logger;
import org.infinispan.Cache;
import org.infinispan.configuration.cache.CacheMode;
import org.infinispan.configuration.cache.Configuration;
import org.infinispan.configuration.cache.ConfigurationBuilder;
import org.infinispan.configuration.global.GlobalConfiguration;
import org.infinispan.configuration.global.GlobalConfigurationBuilder;
import org.infinispan.manager.DefaultCacheManager;
import org.infinispan.transaction.lookup.JBossStandaloneJTAManagerLookup;
import org.infinispan.transaction.lookup.TransactionManagerLookup;

/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

/**
 *
 * @author yeli
 */
public class CacheWrite {
    private static Logger logger = Logger.getLogger(CacheWrite.class);

    public static final String MRID_PREFIX = "test-meas-";
    /**
     * ID start range for measurements managed by this adapter, default is 1
     */
    int startIndex = 1;
    /**
     * ID end range for measurements managed by this adapter, default is 500
     */
    int endIndex = 50;
    /**
     * Randomizer for values
     */
    Random rand = new Random();
    static Cache cache;
    static int membersOnStart = 0;

    Map<String, MeasurementValue> measurements;
    Double measurement;
    
    public static void main(String[] args) {
        final CacheWrite writer = new CacheWrite();
        if (membersOnStart > 1) {
            writer.init();
            Timer timer = new Timer();
            timer.scheduleAtFixedRate(new TimerTask() { 
                @Override
                public void run() {
                    writer.update();
                }
            }, 0, 2000);
        }
    }

    public CacheWrite() {
        GlobalConfiguration gc = new GlobalConfigurationBuilder()
                .transport().defaultTransport()
                .clusterName("test cluster ").build();
        gc = new GlobalConfigurationBuilder().read(gc)
                .transport().addProperty(
                "configurationFile", "my-jgroups-tcp.xml").build();

        TransactionManagerLookup txLookup = new JBossStandaloneJTAManagerLookup();
        Configuration c = new ConfigurationBuilder().clustering()
                .cacheMode(CacheMode.REPL_SYNC).stateTransfer()
                .fetchInMemoryState(true).invocationBatching().enable()
                .locking().useLockStriping(false).transaction()
                .transactionManagerLookup(txLookup)/*.lockingMode(
                LockingMode.PESSIMISTIC)*/.build();

        logger.info("Create cache manager ...");

        DefaultCacheManager manager = new DefaultCacheManager(gc, c, false);
        manager.start();

        logger.info("get cache...");
        Configuration config = new ConfigurationBuilder().read(
                manager.getDefaultCacheConfiguration())
                .storeAsBinary().enabled(false).build();
        config = new ConfigurationBuilder().read(config)
                .clustering().stateTransfer()
                .fetchInMemoryState(true).build();
        config = new ConfigurationBuilder().read(config)
                .clustering().stateTransfer().timeout(240000).build();
        manager.defineConfiguration("test cache", config);
        cache = manager.getCache("test cache");
        logger.info("memebers " + manager.getMembers());
        membersOnStart = manager.getMembers().size();
    }

    void init()
    {
        logger.info("Initializing measurements");

        measurements = new HashMap<String, MeasurementValue>(endIndex - startIndex + 1);
        DoubleValue value = null;

        for (int i=startIndex; i <= endIndex; i++)
        {
            value  = new DoubleValue();
            value.setMRID(MRID_PREFIX + i);
            updateValue(value);
            measurements.put(value.getMRID(), value);
        }
    }


    final void updateValue(DoubleValue analog)
    {
        analog.setReading(rand.nextDouble() * 200.0 - 100.0);
    }

    void update()
    {
        try
        {
            logger.info("updating...");
            long timeBefore = System.currentTimeMillis();
            cache.putAll(measurements);
            long time = System.currentTimeMillis() - timeBefore;
            logger.info("Updated " + measurements.size() + " double values in "
                    + time + " milliseconds");
        }
        catch(Exception ex)
        {
            ex.printStackTrace();
        }
    }
}
